@@ -1,12 +1,12 @@
 /*
  * sha.c: routines to compute SHA-1/224/256/384/512 digests
  *
- * Ref: NIST FIPS PUB 180-2 Secure Hash Standard
+ * Ref: NIST FIPS PUB 180-4 Secure Hash Standard
  *
  * Copyright (C) 2003-2014 Mark Shelor, All Rights Reserved
  *
- * Version: 5.87
- * Mon Feb 17 16:42:02 MST 2014
+ * Version: 5.89
+ * Sat Apr 19 05:14:48 MST 2014
  *
  */
 
@@ -93,7 +93,7 @@ static void sha1(SHA *s, UCHR *block)		/* SHA-1 transform */
 	SHA32_SCHED(W, block);
 
 /*
- * Use SHA-1 alternate method from FIPS PUB 180-2 (ref. 6.1.3)
+ * Use SHA-1 alternate method from FIPS PUB 180-4 (ref. 6.1.3)
  *
  * To improve performance, unroll the loop and consolidate assignments
  * by changing the roles of variables "a" through "e" at each step.
@@ -273,56 +273,82 @@ static UCHR *statecpy(SHA *s, UCHR *buf)
 			*p32++ = memw32(buf);
 	else
 		for (i = 0; i < 8; i++, buf += 8)
-			*p64++ = ((W64) memw32(buf) << 32) +
+			*p64++ = (((W64)memw32(buf) << 16) << 16) +
 					memw32(buf+4);
 	return(buf);
 }
 
-#define SHA_INIT(algo, transform) 					\
+#define SHA_INIT(s, algo, transform) 					\
 	do {								\
-		memset(s, 0, sizeof(SHA));				\
+		int allocated = s->allocated;				\
+		SHA_Zero(s, 1, SHA);					\
 		s->alg = algo; s->sha = sha ## transform;		\
-		memcpy(s->H, H0 ## algo, sizeof(H0 ## algo));		\
+		SHA_Copy(H0 ## algo, s->H, sizeof(H0 ## algo), char);	\
 		s->blocksize = SHA ## algo ## _BLOCK_BITS;		\
 		s->digestlen = SHA ## algo ## _DIGEST_BITS >> 3;	\
+		s->allocated = allocated;					\
 	} while (0)
 
-/* sharewind: re-initializes the digest object */
+/* sharewind: resets digest object */
 static void sharewind(SHA *s)
 {
-	if      (s->alg == SHA1)   SHA_INIT(1, 1);
-	else if (s->alg == SHA224) SHA_INIT(224, 256);
-	else if (s->alg == SHA256) SHA_INIT(256, 256);
-	else if (s->alg == SHA384) SHA_INIT(384, 512);
-	else if (s->alg == SHA512) SHA_INIT(512, 512);
-	else if (s->alg == SHA512224) SHA_INIT(512224, 512);
-	else if (s->alg == SHA512256) SHA_INIT(512256, 512);
+	if      (s->alg == SHA1)   SHA_INIT(s, 1, 1);
+	else if (s->alg == SHA224) SHA_INIT(s, 224, 256);
+	else if (s->alg == SHA256) SHA_INIT(s, 256, 256);
+	else if (s->alg == SHA384) SHA_INIT(s, 384, 512);
+	else if (s->alg == SHA512) SHA_INIT(s, 512, 512);
+	else if (s->alg == SHA512224) SHA_INIT(s, 512224, 512);
+	else if (s->alg == SHA512256) SHA_INIT(s, 512256, 512);
 }
 
-/* shaopen: creates a new digest object */
-static SHA *shaopen(int alg)
+/* shainit: initializes digest object */
+static SHA *shainit(SHA *s, int alg)
 {
-	SHA *s = NULL;
-
+	if (alg >= SHA384 && !sha_384_512)
+		return(NULL);
 	if (alg != SHA1 && alg != SHA224 && alg != SHA256 &&
 		alg != SHA384    && alg != SHA512 &&
 		alg != SHA512224 && alg != SHA512256)
 		return(NULL);
-	if (alg >= SHA384 && !sha_384_512)
-		return(NULL);
-	SHA_newz(0, s, 1, SHA);
-	if (s == NULL)
-		return(NULL);
 	s->alg = alg;
 	sharewind(s);
 	return(s);
 }
 
-/* shaclose: de-allocates digest object */
+/* shaopen: creates new digest object */
+static SHA *shaopen(int alg)
+{
+	SHA *s;
+
+	SHA_newz(0, s, 1, SHA);
+	if (s == NULL)
+		return(NULL);
+	if (shainit(s, alg) == NULL) {
+		SHA_free(s);
+		return(NULL);
+	}
+	s->allocated = 1;
+	return(s);
+}
+
+/* shadup: duplicates digest object */
+static SHA *shadup(SHA *s)
+{
+	SHA *p;
+
+	SHA_new(0, p, 1, SHA);
+	if (p == NULL)
+		return(NULL);
+	SHA_Copy(s, p, 1, SHA);
+	p->allocated = 1;
+	return(p);
+}
+
+/* shaclose: frees memory used by digest object */
 static int shaclose(SHA *s)
 {
-	if (s != NULL) {
-		memset(s, 0, sizeof(SHA));
+	if (s->allocated) {
+		s->allocated = 0;
 		SHA_free(s);
 	}
 	return(0);
@@ -339,13 +365,13 @@ static ULNG shadirect(UCHR *bitstr, ULNG bitcnt, SHA *s)
 		bitcnt -= s->blocksize;
 	}
 	if (bitcnt > 0) {
-		memcpy(s->block, bitstr, NBYTES(bitcnt));
+		SHA_Copy(bitstr, s->block, NBYTES(bitcnt), char);
 		s->blockcnt = bitcnt;
 	}
 	return(savecnt);
 }
 
-/* shabytes: updates state for byte-aligned input data */
+/* shabytes: updates state for byte-aligned data in s->block */
 static ULNG shabytes(UCHR *bitstr, ULNG bitcnt, SHA *s)
 {
 	UINT offset;
@@ -355,20 +381,20 @@ static ULNG shabytes(UCHR *bitstr, ULNG bitcnt, SHA *s)
 	offset = s->blockcnt >> 3;
 	if (s->blockcnt + bitcnt >= s->blocksize) {
 		nbits = s->blocksize - s->blockcnt;
-		memcpy(s->block+offset, bitstr, nbits>>3);
+		SHA_Copy(bitstr, s->block+offset, nbits>>3, char);
 		bitcnt -= nbits;
 		bitstr += (nbits >> 3);
 		s->sha(s, s->block), s->blockcnt = 0;
 		shadirect(bitstr, bitcnt, s);
 	}
 	else {
-		memcpy(s->block+offset, bitstr, NBYTES(bitcnt));
+		SHA_Copy(bitstr, s->block+offset, NBYTES(bitcnt), char);
 		s->blockcnt += bitcnt;
 	}
 	return(savecnt);
 }
 
-/* shabits: updates state for bit-aligned input data */
+/* shabits: updates state for bit-aligned data in s->block */
 static ULNG shabits(UCHR *bitstr, ULNG bitcnt, SHA *s)
 {
 	UINT i;
@@ -446,6 +472,8 @@ static void shafinish(SHA *s)
 	s->sha(s, s->block);
 }
 
+#define shadigest(state)	digcpy(state)
+
 /* xmap: translation map for hexadecimal encoding */
 static char xmap[] =
 	"0123456789abcdef";
@@ -481,7 +509,7 @@ static void encbase64(UCHR *in, int n, char *out)
 	out[0] = '\0';
 	if (n < 1 || n > 3)
 		return;
-	memcpy(byte, in, n);
+	SHA_Copy(in, byte, n, UCHR);
 	out[0] = bmap[byte[0] >> 2];
 	out[1] = bmap[((byte[0] & 0x03) << 4) | (byte[1] >> 4)];
 	out[2] = bmap[((byte[1] & 0x0f) << 2) | (byte[2] >> 6)];
@@ -509,94 +537,61 @@ static char *shabase64(SHA *s)
 	return(s->base64);
 }
 
-/* shadup: duplicates current digest object */
-static SHA *shadup(SHA *s)
-{
-	SHA *p;
-
-	SHA_new(0, p, 1, SHA);
-	if (p == NULL)
-		return(NULL);
-	memcpy(p, s, sizeof(SHA));
-	return(p);
-}
-
-/* hmacopen: creates a new HMAC-SHA digest object */
-static HMAC *hmacopen(int alg, UCHR *key, UINT keylen)
+/* hmacinit: initializes HMAC-SHA digest object */
+static HMAC *hmacinit(HMAC *h, int alg, UCHR *key, UINT keylen)
 {
 	UINT i;
-	HMAC *h;
+	SHA ksha;
 
-	SHA_newz(0, h, 1, HMAC);
-	if (h == NULL)
+	SHA_Zero(h, 1, HMAC);
+	if (shainit(&h->isha, alg) == NULL)
 		return(NULL);
-	if ((h->isha = shaopen(alg)) == NULL) {
-		SHA_free(h);
+	if (shainit(&h->osha, alg) == NULL)
 		return(NULL);
-	}
-	if ((h->osha = shaopen(alg)) == NULL) {
-		shaclose(h->isha);
-		SHA_free(h);
-		return(NULL);
-	}
-	if (keylen <= h->osha->blocksize / 8)
-		memcpy(h->key, key, keylen);
+	if (keylen <= h->osha.blocksize / 8)
+		SHA_Copy(key, h->key, keylen, char);
 	else {
-		if ((h->ksha = shaopen(alg)) == NULL) {
-			shaclose(h->isha);
-			shaclose(h->osha);
-			SHA_free(h);
+		if (shainit(&ksha, alg) == NULL)
 			return(NULL);
-		}
-		shawrite(key, keylen * 8, h->ksha);
-		shafinish(h->ksha);
-		memcpy(h->key, digcpy(h->ksha), h->ksha->digestlen);
-		shaclose(h->ksha);
+		shawrite(key, keylen * 8, &ksha);
+		shafinish(&ksha);
+		SHA_Copy(digcpy(&ksha), h->key, ksha.digestlen, char);
 	}
-	for (i = 0; i < h->osha->blocksize / 8; i++)
+	h->digestlen = h->osha.digestlen;
+	for (i = 0; i < h->osha.blocksize / 8; i++)
 		h->key[i] ^= 0x5c;
-	shawrite(h->key, h->osha->blocksize, h->osha);
-	for (i = 0; i < h->isha->blocksize / 8; i++)
+	shawrite(h->key, h->osha.blocksize, &h->osha);
+	for (i = 0; i < h->isha.blocksize / 8; i++)
 		h->key[i] ^= (0x5c ^ 0x36);
-	shawrite(h->key, h->isha->blocksize, h->isha);
-	memset(h->key, 0, sizeof(h->key));
+	shawrite(h->key, h->isha.blocksize, &h->isha);
+	SHA_Zero(h->key, sizeof(h->key), char);
 	return(h);
 }
 
 /* hmacwrite: triggers a state update using data in bitstr/bitcnt */
 static ULNG hmacwrite(UCHR *bitstr, ULNG bitcnt, HMAC *h)
 {
-	return(shawrite(bitstr, bitcnt, h->isha));
+	return(shawrite(bitstr, bitcnt, &h->isha));
 }
 
 /* hmacfinish: computes final digest state */
 static void hmacfinish(HMAC *h)
 {
-	shafinish(h->isha);
-	shawrite(digcpy(h->isha), h->isha->digestlen * 8, h->osha);
-	shaclose(h->isha);
-	shafinish(h->osha);
+	shafinish(&h->isha);
+	shawrite(digcpy(&h->isha), h->isha.digestlen * 8, &h->osha);
+	shafinish(&h->osha);
 }
 
+#define hmacdigest(h)	digcpy(&(h)->osha)
+
 /* hmachex: returns pointer to digest (hexadecimal) */
 static char *hmachex(HMAC *h)
 {
-	return(shahex(h->osha));
+	return(shahex(&h->osha));
 }
 
 /* hmacbase64: returns pointer to digest (Base 64) */
 static char *hmacbase64(HMAC *h)
 {
-	return(shabase64(h->osha));
-}
-
-/* hmacclose: de-allocates digest object */
-static int hmacclose(HMAC *h)
-{
-	if (h != NULL) {
-		shaclose(h->osha);
-		memset(h, 0, sizeof(HMAC));
-		SHA_free(h);
-	}
-	return(0);
+	return(shabase64(&h->osha));
 }
