@@ -1,12 +1,12 @@
 /*
  * sha.h: header file for SHA-1/224/256/384/512 routines
  *
- * Ref: NIST FIPS PUB 180-2 Secure Hash Standard
+ * Ref: NIST FIPS PUB 180-4 Secure Hash Standard
  *
  * Copyright (C) 2003-2014 Mark Shelor, All Rights Reserved
  *
- * Version: 5.86
- * Thu Jan 30 08:24:28 MST 2014
+ * Version: 5.89
+ * Sat Apr 19 05:14:48 MST 2014
  *
  */
 
@@ -15,6 +15,12 @@
 
 #include <limits.h>
 
+#define SHA_new		New
+#define SHA_newz	Newz
+#define SHA_free	Safefree
+#define SHA_Copy	Copy
+#define SHA_Zero	Zero
+
 #define SHA32_MAX	4294967295U
 
 #define SHA32_SHR(x, n)	((x) >> (n))
@@ -79,10 +85,10 @@
 
 #if defined(BYTEORDER) && (BYTEORDER & 0xffff) == 0x4321
 	#if defined(SHA32_ALIGNED)
-		#define SHA32_SCHED(W, b)	memcpy(W, b, 64)
+		#define SHA32_SCHED(W, b)	SHA_Copy(b, W, 64, char)
 	#endif
 	#if defined(SHA64) && defined(SHA64_ALIGNED)
-		#define SHA64_SCHED(W, b)	memcpy(W, b, 128)
+		#define SHA64_SCHED(W, b)	SHA_Copy(b, W, 128, char)
 	#endif
 #endif
 
@@ -102,30 +108,6 @@
 			(SHA64) b[6] <<  8 | (SHA64) b[7]; }
 #endif
 
-#define SHA_new		New
-#define SHA_newz	Newz
-#define SHA_free	Safefree
-
-#ifdef SHA_PerlIO
-	#define SHA_FILE		PerlIO
-	#define SHA_stdin()		PerlIO_stdin()
-	#define SHA_stdout()		PerlIO_stdout()
-	#define SHA_open		PerlIO_open
-	#define SHA_close		PerlIO_close
-	#define SHA_fprintf		PerlIO_printf
-	#define SHA_feof		PerlIO_eof
-	#define SHA_getc		PerlIO_getc
-#else
-	#define SHA_FILE		FILE
-	#define SHA_stdin()		stdin
-	#define SHA_stdout()		stdout
-	#define SHA_open		fopen
-	#define SHA_close		fclose
-	#define SHA_fprintf		fprintf
-	#define SHA_feof		feof
-	#define SHA_getc		fgetc
-#endif
-
 #define SHA1		1
 #define SHA224		224
 #define SHA256		256
@@ -173,12 +155,13 @@ typedef struct SHA {
 	int digestlen;
 	char hex[SHA_MAX_HEX_LEN+1];
 	char base64[SHA_MAX_BASE64_LEN+1];
+	int allocated;
 } SHA;
 
 typedef struct {
-	SHA *ksha;
-	SHA *isha;
-	SHA *osha;
+	SHA isha;
+	SHA osha;
+	int digestlen;
 	unsigned char key[SHA_MAX_BLOCK_BITS/8];
 } HMAC;
 
